@@ -1,9 +1,12 @@
-from testfixtures import Comparison as C, TempDirectory, compare
-from testfixtures.compat import PY2, PY3, exception_module
-from testfixtures.tests.sample1 import SampleClassA, a_function
 from unittest import TestCase
 import sys
 
+from testfixtures import Comparison as C, TempDirectory, compare, diff
+from testfixtures.compat import PY2, PY3, exception_module
+from testfixtures.shouldraise import ShouldAssert
+from testfixtures.tests.sample1 import SampleClassA, a_function
+import pytest
+
 
 class AClass:
 
@@ -29,9 +32,6 @@ class WeirdException(Exception):
 class X(object):
     __slots__ = ['x']
 
-    def __repr__(self):
-        return '<X>'
-
 
 class FussyDefineComparison(object):
 
@@ -47,6 +47,12 @@ class FussyDefineComparison(object):
         return not self == other  # pragma: no cover
 
 
+def compare_repr(obj, expected):
+    actual = diff(expected, repr(obj))
+    if actual:  # pragma: no cover
+        raise AssertionError(actual)
+
+
 class TestC(TestCase):
 
     def test_example(self):
@@ -114,260 +120,181 @@ class TestC(TestCase):
     def test_example_dont_use_c_wrappers_on_both_sides(self):
         # NB: don't use C wrappers on both sides!
         e = ValueError('some message')
-        try:
-            self.assertEqual(C(e), C(e))
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(
-                e.args,
-                (("<C(failed):{mod}.ValueError>wrong type</C> != \n"
-                  "  <C:{mod}.ValueError>\n"
-                  "  args:('some message',)\n"
-                  "  </C>"
-                  ).format(mod=exception_module), ))
-        else:
-            self.fail('No exception raised!')
+        x, y = C(e), C(e)
+        assert x != y
+        compare_repr(x, "<C(failed):{mod}.ValueError>wrong type</C>".format(
+                            mod=exception_module))
+        compare_repr(
+            y,
+            "<C:{mod}.ValueError>args: ('some message',)</C>".format(
+                mod=exception_module)
+        )
 
     def test_repr_module(self):
-        self.assertEqual(repr(C('datetime')), '<C:datetime>')
+        compare_repr(C('datetime'), '<C:datetime>')
 
     def test_repr_class(self):
-        self.assertEqual(
-            repr(C('testfixtures.tests.sample1.SampleClassA')),
-            '<C:testfixtures.tests.sample1.SampleClassA>'
-            )
+        compare_repr(C('testfixtures.tests.sample1.SampleClassA'),
+                     '<C:testfixtures.tests.sample1.SampleClassA>')
 
     def test_repr_function(self):
-        self.assertEqual(
-            repr(C('testfixtures.tests.sample1.z')),
-            '<C:testfixtures.tests.sample1.z>'
-            )
+        compare_repr(C('testfixtures.tests.sample1.z'),
+                     '<C:testfixtures.tests.sample1.z>')
 
     def test_repr_instance(self):
-        self.assertEqual(
-            repr(C(SampleClassA('something'))),
-            "\n"
-            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
-            "  args:('something',)\n"
-            "  </C>"
-            )
+        compare_repr(C(SampleClassA('something')),
+                     "<C:testfixtures.tests.sample1.SampleClassA>"
+                     "args: ('something',)"
+                     "</C>"
+                     )
 
     def test_repr_exception(self):
-        self.assertEqual(
-            repr(C(ValueError('something'))),
-            ("\n"
-             "  <C:{0}.ValueError>\n"
-             "  args:('something',)\n"
-             "  </C>"
-             ).format(exception_module))
+        compare_repr(C(ValueError('something')),
+                     ("<C:{0}.ValueError>args: ('something',)</C>"
+                      ).format(exception_module))
 
     def test_repr_exception_not_args(self):
-        r = repr(C(WeirdException(1, 2)))
-
         if sys.version_info >= (3, 2, 4):
             # in PY3, even args that aren't set still appear to be there
-            args = "  args:(1, 2)\n"
+            args = "args: (1, 2)\n"
         else:
-            args = "  args:()\n"
+            args = "args: ()\n"
 
-        self.assertEqual(
-            r,
-            "\n"
-            "  <C:testfixtures.tests.test_comparison.WeirdException>\n"
+        compare_repr(
+            C(WeirdException(1, 2)),
+            "\n<C:testfixtures.tests.test_comparison.WeirdException>\n"
             + args +
-            "  x:1\n"
-            "  y:2\n"
-            "  </C>"
-            )
+            "x: 1\n"
+            "y: 2\n"
+            "</C>"
+        )
 
     def test_repr_class_and_vars(self):
-        self.assertEqual(
-            repr(C(SampleClassA, {'args': (1,)})),
-            "\n"
-            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
-            "  args:(1,)\n"
-            "  </C>"
-            )
+        compare_repr(
+            C(SampleClassA, {'args': (1,)}),
+            "<C:testfixtures.tests.sample1.SampleClassA>args: (1,)</C>"
+        )
 
     def test_repr_nested(self):
-        self.assertEqual(
-            repr(C(SampleClassA, y=C(AClass), z=C(BClass(1, 2)))),
+        compare_repr(
+            C(SampleClassA, y=C(AClass), z=C(BClass(1, 2))),
             "\n"
-            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
-            "  y:<C:testfixtures.tests.test_comparison.AClass>\n"
-            "  z:\n"
-            "    <C:testfixtures.tests.test_comparison.BClass>\n"
-            "    x:1\n"
-            "    y:2\n"
-            "    </C>\n"
-            "  </C>"
+            "<C:testfixtures.tests.sample1.SampleClassA>\n"
+            "y: <C:testfixtures.tests.test_comparison.AClass>\n"
+            "z: \n"
+            "  <C:testfixtures.tests.test_comparison.BClass>\n"
+            "  x: 1\n"
+            "  y: 2\n"
+            "  </C>\n"
+            "</C>"
             )
 
     def test_repr_failed_wrong_class(self):
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass',
-                  x=1, y=2),
-                BClass(1, 2)
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(e.args, ((
-                "<C(failed):testfixtures.tests.test_comparison.AClass>"
-                "wrong type</C> != <BClass>"
-                ), ))
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass', x=1, y=2)
+        assert c != BClass(1, 2)
+        compare_repr(c,
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>"
+                     "wrong type</C>"
+                     )
 
     def test_repr_failed_all_reasons_in_one(self):
-        if PY3:
-            expected = (
-                "\n  <C(failed):testfixtures.tests.test_com[79 chars] </C>"
-                " != <AClass>",
-                )
-        else:
-            expected = (
-                "\n"
-                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                "  x:1 not in Comparison\n"
-                "  y:5 != 2\n"
-                "  z:'missing' not in other\n"
-                "  </C> != <AClass>",
-                )
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass',
-                  y=5, z='missing'),
-                AClass(1, 2)
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(e.args, expected)
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass',
+              y=5, z='missing')
+        assert c != AClass(1, 2)
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes in Comparison but not actual:\n"
+                     "'z': 'missing'\n\n"
+                     "attributes in actual but not Comparison:\n"
+                     "'x': 1\n\n"
+                     "attributes differ:\n"
+                     "'y': 5 (Comparison) != 2 (actual)\n"
+                     "</C>",
+                     )
 
     def test_repr_failed_not_in_other(self):
-        if PY3:
-            expected = (
-                "\n  <C(failed):testfixtures.tests.test_com[39 chars] </C>"
-                " != <AClass>",
-                )
-        else:
-            expected = (
-                "\n"
-                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                "  z:(3,) not in other\n"
-                "  </C> != <AClass>",
-                )
-        # use single element tuple to check %
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass',
-                  x=1, y=2, z=(3, )),
-                AClass(1, 2)
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(e.args, expected)
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass',
+              x=1, y=2, z=(3, ))
+        assert c != AClass(1, 2)
+        compare_repr(c ,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['x', 'y']\n\n"
+                     "attributes in Comparison but not actual:\n"
+                     "'z': (3,)\n"
+                     "</C>",
+                     )
 
     def test_repr_failed_not_in_self_strict(self):
-        # use single element tuple to check %
-        if PY3:
-            expected = (
-                "\n  <C(failed):testfixtures.tests.test_com[44 chars] </C>"
-                " != <AClass>",
-                )
-        else:
-            expected = (
-                "\n"
-                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                "  x:(1,) not in Comparison\n"
-                "  </C> != <AClass>",
-                )
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass', y=2),
-                AClass((1, ), 2)
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(e.args, expected)
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass', y=2)
+        assert c != AClass((1, ), 2)
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['y']\n\n"
+                     "attributes in actual but not Comparison:\n"
+                     "'x': (1,)\n"
+                     "</C>",
+                     )
 
     def test_repr_failed_not_in_self_not_strict(self):
-        if PY3:
-            expected = (
-                "\n  <C(failed):testfixtures.tests.test_com[39 chars] </C>"
-                " != <AClass>",
-                )
-        else:
-            expected = (
-                "\n"
-                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                "  z:(3,) not in other\n"
-                "  </C> != <AClass>",
-                )
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass',
-                  x=1, y=2, z=(3, )),
-                AClass(1, 2)
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(e.args, expected)
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass',
+              x=1, y=2, z=(3, ))
+        assert c != AClass(1, 2)
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['x', 'y']\n\n"
+                     "attributes in Comparison but not actual:\n"
+                     "'z': (3,)\n"
+                     "</C>",
+                     )
 
     def test_repr_failed_one_attribute_not_equal(self):
-        # use single element tuple to check %
-        try:
-            self.assertEqual(
-                C('testfixtures.tests.test_comparison.AClass', x=1, y=(2, )),
-                AClass(1, (3, ))
-                )
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(
-                e.args,
-                ("\n"
-                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                 "  y:(2,) != (3,)\n"
-                 "  </C> != <AClass>",
-                 ))
-        else:
-            self.fail('No exception raised!')
+        c = C('testfixtures.tests.test_comparison.AClass', x=1, y=(2, ))
+        assert c != AClass(1, (3, ))
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['x']\n\n"
+                     "attributes differ:\n"
+                     "'y': (2,) (Comparison) != (3,) (actual)\n"
+                     "</C>",
+                     )
 
     def test_repr_failed_nested(self):
         left_side = [C(AClass, x=1, y=2),
-                     C(BClass, x=C(AClass, x=1), y=C(AClass))]
+                     C(BClass, x=C(AClass, x=1, y=2), y=C(AClass))]
         right_side = [AClass(1, 3), AClass(1, 3)]
 
         # do the comparison
         left_side == right_side
 
-        self.assertEqual(
+        compare_repr(
+            left_side,
             "[\n"
-            "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-            "  y:2 != 3\n"
-            "  </C>, \n"
-            "  <C:testfixtures.tests.test_comparison.BClass>\n"
-            "  x:\n"
-            "    <C:testfixtures.tests.test_comparison.AClass>\n"
-            "    x:1\n"
-            "    </C>\n"
-            "  y:<C:testfixtures.tests.test_comparison.AClass>\n"
-            "  </C>]",
-            repr(left_side)
-            )
+            "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+            "attributes same:\n"
+            "['x']\n\n"
+            "attributes differ:\n"
+            "'y': 2 (Comparison) != 3 (actual)\n"
+            "</C>, \n"
+            "<C:testfixtures.tests.test_comparison.BClass>\n"
+            "x: \n"
+            "  <C:testfixtures.tests.test_comparison.AClass>\n"
+            "  x: 1\n"
+            "  y: 2\n"
+            "  </C>\n"
+            "y: <C:testfixtures.tests.test_comparison.AClass>\n"
+            "</C>]"
+        )
 
-        self.assertEqual(
-            "[<AClass>, <AClass>]",
-            repr(right_side)
-            )
+        compare_repr(right_side, "[<AClass>, <AClass>]")
 
     def test_repr_failed_nested_failed(self):
         left_side = [C(AClass, x=1, y=2),
@@ -380,60 +307,55 @@ class TestC(TestCase):
         # do the comparison
         left_side == right_side
 
-        self.assertEqual(
+        compare_repr(
+            left_side,
             "[\n"
-            "  <C:testfixtures.tests.test_comparison.AClass>\n"
-            "  x:1\n"
-            "  y:2\n"
-            "  </C>, \n"
-            "  <C(failed):testfixtures.tests.test_comparison.BClass>\n"
-            "  y:\n"
-            "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-            "  x:1 not in Comparison\n"
-            "  y:2 not in Comparison\n"
-            "  z:2 not in other\n"
-            "  </C> != <AClass>\n"
-            "  </C>]",
-            repr(left_side)
-            )
+            "<C:testfixtures.tests.test_comparison.AClass>\n"
+            "x: 1\n"
+            "y: 2\n"
+            "</C>, \n"
+            "<C(failed):testfixtures.tests.test_comparison.BClass>\n"
+            "attributes same:\n"
+            "['x']\n\n"
+            "attributes differ:\n"
+            "'y': \n"
+            "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+            "attributes in Comparison but not actual:\n"
+            "'z': 2\n\n"
+            "attributes in actual but not Comparison:\n"
+            "'x': 1\n"
+            "'y': 2\n"
+            "</C> (Comparison) != <AClass> (actual)\n"
+            "</C>]",
+        )
 
-        self.assertEqual(
-            '[<AClass>, <BClass>]',
-            repr(right_side)
-            )
+        compare_repr(right_side, '[<AClass>, <BClass>]')
 
     def test_repr_failed_passed_failed(self):
         c = C('testfixtures.tests.test_comparison.AClass', x=1, y=2)
+        assert c != AClass(1, 3)
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['x']\n\n"
+                     "attributes differ:\n"
+                     "'y': 2 (Comparison) != 3 (actual)\n"
+                     "</C>",
+                     )
 
-        try:
-            self.assertEqual(c, AClass(1, 3))
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(
-                e.args,
-                ("\n"
-                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                 "  y:2 != 3\n"
-                 "  </C> != <AClass>",
-                 ))
-        else:
-            self.fail('No exception raised!')
+        assert c == AClass(1, 2)
 
-        self.assertEqual(c, AClass(1, 2))
-
-        try:
-            self.assertEqual(c, AClass(3, 2))
-        except Exception as e:
-            self.failUnless(isinstance(e, AssertionError))
-            self.assertEqual(
-                e.args,
-                ("\n"
-                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
-                 "  x:1 != 3\n"
-                 "  </C> != <AClass>",
-                 ))
-        else:
-            self.fail('No exception raised!')
+        assert c != AClass(3, 2)
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.AClass>\n"
+                     "attributes same:\n"
+                     "['y']\n\n"
+                     "attributes differ:\n"
+                     "'x': 1 (Comparison) != 3 (actual)\n"
+                     "</C>",
+                     )
 
     def test_first(self):
         self.assertEqual(
@@ -549,22 +471,28 @@ class TestC(TestCase):
         if PY3:
             c = C('io.TextIOWrapper', name=path, mode='r', closed=False,
                   strict=False)
-            self.assertNotEqual(f, c)
-            compare(repr(c),
-                    "\n"
-                    "  <C(failed):_io.TextIOWrapper>\n"
-                    "  closed:False != True\n"
-                    "  </C>",
-                    )
+            assert f != c
+            compare_repr(c,
+                         "\n"
+                         "<C(failed):_io.TextIOWrapper>\n"
+                         "attributes same:\n"
+                         "['mode', 'name']\n\n"
+                         "attributes differ:\n"
+                         "'closed': False (Comparison) != True (actual)\n"
+                         "</C>",
+                         )
         else:
             c = C(file, name=path, mode='r', closed=False, strict=False)
-            self.assertNotEqual(f, c)
-            compare(repr(c),
-                    "\n"
-                    "  <C(failed):__builtin__.file>\n"
-                    "  closed:False != True\n"
-                    "  </C>",
-                    )
+            assert f != c
+            compare_repr(c,
+                         "\n"
+                         "<C(failed):__builtin__.file>\n"
+                         "attributes same:\n"
+                         "['mode', 'name']\n\n"
+                         "attributes differ:\n"
+                         "'closed': False (Comparison) != True (actual)\n"
+                         "</C>",
+                         )
 
     def test_file_same(self):
         with TempDirectory() as d:
@@ -584,49 +512,44 @@ class TestC(TestCase):
                 )
 
     def test_no___dict___strict(self):
-        x = X()
-        try:
-            self.assertEqual(
-                C(X, x=1),
-                x
-                )
-        except TypeError as e:
-            self.assertEqual(e.args, (
-                '<X> does not support vars() so cannot do strict comparison',
-                ))
-        else:
-            self.fail('No exception raised!')
+        c = C(X, x=1)
+        assert c != X()
+        compare_repr(c, "\n"
+                        "<C(failed):testfixtures.tests.test_comparison.X>\n"
+                        "attributes in Comparison but not actual:\n"
+                        "'x': 1\n"
+                        "</C>")
 
     def test_no___dict___not_strict_same(self):
         x = X()
         x.x = 1
         self.assertEqual(C(X, x=1, strict=False), x)
 
+    def test_no___dict___not_strict_missing_attr(self):
+        c = C(X, x=1, strict=False)
+        assert c != X()
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.X>\n"
+                     "attributes in Comparison but not actual:\n"
+                     "'x': 1\n"
+                     "</C>",
+                     )
+
     def test_no___dict___not_strict_different(self):
-        if PY3:
-            expected = (
-                "\n  <C(failed):testfixtures.tests.test_com[42 chars] </C>"
-                " != <X>",
-                )
-        else:
-            expected = (
-                "\n"
-                "  <C(failed):testfixtures.tests.test_comparison.X>\n"
-                "  x:1 != 2\n"
-                "  y:2 not in other\n"
-                "  </C> != <X>",
-                )
         x = X()
         x.x = 2
-        try:
-            self.assertEqual(
-                C(X, x=1, y=2, strict=False),
-                x
-                )
-        except AssertionError as e:
-            compare(e.args, expected)
-        else:
-            self.fail('No exception raised!')
+        c = C(X, x=1, y=2, strict=False)
+        assert c != x
+        compare_repr(c,
+                     "\n"
+                     "<C(failed):testfixtures.tests.test_comparison.X>\n"
+                     "attributes in Comparison but not actual:\n"
+                     "'y': 2\n\n"
+                     "attributes differ:\n"
+                     "'x': 1 (Comparison) != 2 (actual)\n"
+                     "</C>",
+                     )
 
     def test_compared_object_defines_eq(self):
         # If an object defines eq, such as Django instances,
@@ -663,47 +586,16 @@ class TestC(TestCase):
         # comparison representation afterwards
         c = C(Annoying, eq_called=1)
         c == Annoying()
-        self.assertEqual(
-            repr(c),
-            '\n  <C(failed):testfixtures.tests.test_comparison.Annoying>\n'
-            '  eq_called:1 != 0\n'
-            '  </C>')
-
-    def test_compared_object_class_attributes(self):
-
-        class Classy(object):
-            x = 1
-            y = 2
-
-        self.assertEqual(C(Classy, x=1, y=2), Classy())
-
-        c = C(Classy, x=1, y=1)
-        self.assertNotEqual(c, Classy())
-        self.assertEqual(
-            repr(c),
-            '\n  <C(failed):testfixtures.tests.test_comparison.Classy>\n'
-            '  y:1 != 2\n'
-            '  </C>')
-
-        ce = C(Classy, x=1, y=1)
-        ca = Classy()
-        ca.y = 1
-        self.assertEqual(ce, ca)
-
-        ce = C(Classy, x=1, y=2)
-        ca = Classy()
-        ca.y = 1
-        self.assertNotEqual(ce, ca)
-        self.assertEqual(
-            repr(ce),
-            '\n  <C(failed):testfixtures.tests.test_comparison.Classy>\n'
-            '  y:2 != 1\n'
-            '  </C>')
+        compare_repr(
+            c,
+            '\n<C(failed):testfixtures.tests.test_comparison.Annoying>\n'
+            'attributes differ:\n'
+            "'eq_called': 1 (Comparison) != 0 (actual)\n"
+            '</C>'
+        )
 
     def test_importerror(self):
-        self.failIf(
-            C(ImportError('x')) != ImportError('x')
-            )
+        assert C(ImportError('x')) == ImportError('x')
 
     def test_class_defines_comparison_strictly(self):
         self.assertEqual(
